@@ -6,7 +6,6 @@
 #include "client.hpp"
 #include "modules/sway/ipc/client.hpp"
 #include "util/json.hpp"
-#include "util/sleeper_thread.hpp"
 
 namespace waybar::modules::sway {
 
@@ -18,14 +17,11 @@ class Mode : public ALabel, public sigc::trackable {
 
  private:
   void onEvent(const struct Ipc::ipc_response&);
-  void worker();
 
   std::string      mode_;
   util::JsonParser parser_;
   std::mutex       mutex_;
-
-  util::SleeperThread thread_;
-  Ipc                 ipc_;
+  Ipc              ipc_;
 };
 
 }  // namespace waybar::modules::sway
