@@ -9,20 +9,103 @@
 #include "bar.hpp"
 #include "client.hpp"
 #include "factory.hpp"
+#include "group.hpp"
 #include "wlr-layer-shell-unstable-v1-client-protocol.h"
 
+#ifdef HAVE_SWAY
+#include "modules/sway/bar.hpp"
+#endif
+
 namespace waybar {
 static constexpr const char* MIN_HEIGHT_MSG =
-    "Requested height: {} exceeds the minimum height: {} required by the modules";
+    "Requested height: {} is less than the minimum height: {} required by the modules";
 
 static constexpr const char* MIN_WIDTH_MSG =
-    "Requested width: {} exceeds the minimum width: {} required by the modules";
+    "Requested width: {} is less than the minimum width: {} required by the modules";
 
 static constexpr const char* BAR_SIZE_MSG = "Bar configured (width: {}, height: {}) for output: {}";
 
 static constexpr const char* SIZE_DEFINED =
     "{} size is defined in the config file so it will stay like that";
 
+const Bar::bar_mode_map Bar::PRESET_MODES = {  //
+    {"default",
+     {// Special mode to hold the global bar configuration
+      .layer = bar_layer::BOTTOM,
+      .exclusive = true,
+      .passthrough = false,
+      .visible = true}},
+    {"dock",
+     {// Modes supported by the sway config; see man sway-bar(5)
+      .layer = bar_layer::BOTTOM,
+      .exclusive = true,
+      .passthrough = false,
+      .visible = true}},
+    {"hide",
+     {//
+      .layer = bar_layer::TOP,
+      .exclusive = false,
+      .passthrough = false,
+      .visible = true}},
+    {"invisible",
+     {//
+      .layer = bar_layer::BOTTOM,
+      .exclusive = false,
+      .passthrough = true,
+      .visible = false}},
+    {"overlay",
+     {//
+      .layer = bar_layer::TOP,
+      .exclusive = false,
+      .passthrough = true,
+      .visible = true}}};
+
+const std::string_view Bar::MODE_DEFAULT = "default";
+const std::string_view Bar::MODE_INVISIBLE = "invisible";
+const std::string_view DEFAULT_BAR_ID = "bar-0";
+
+/* Deserializer for enum bar_layer */
+void from_json(const Json::Value& j, bar_layer& l) {
+  if (j == "bottom") {
+    l = bar_layer::BOTTOM;
+  } else if (j == "top") {
+    l = bar_layer::TOP;
+  } else if (j == "overlay") {
+    l = bar_layer::OVERLAY;
+  }
+}
+
+/* Deserializer for struct bar_mode */
+void from_json(const Json::Value& j, bar_mode& m) {
+  if (j.isObject()) {
+    if (auto v = j["layer"]; v.isString()) {
+      from_json(v, m.layer);
+    }
+    if (auto v = j["exclusive"]; v.isBool()) {
+      m.exclusive = v.asBool();
+    }
+    if (auto v = j["passthrough"]; v.isBool()) {
+      m.passthrough = v.asBool();
+    }
+    if (auto v = j["visible"]; v.isBool()) {
+      m.visible = v.asBool();
+    }
+  }
+}
+
+/* Deserializer for JSON Object -> map<string compatible type, Value>
+ * Assumes that all the values in the object are deserializable to the same type.
+ */
+template <typename Key, typename Value,
+          typename = std::enable_if_t<std::is_convertible<std::string_view, Key>::value>>
+void from_json(const Json::Value& j, std::map<Key, Value>& m) {
+  if (j.isObject()) {
+    for (auto it = j.begin(); it != j.end(); ++it) {
+      from_json(*it, m[it.key().asString()]);
+    }
+  }
+}
+
 #ifdef HAVE_GTK_LAYER_SHELL
 struct GLSSurfaceImpl : public BarSurface, public sigc::trackable {
   GLSSurfaceImpl(Gtk::Window& window, struct waybar_output& output) : window_{window} {
@@ -33,6 +116,7 @@ struct GLSSurfaceImpl : public BarSurface, public sigc::trackable {
     gtk_layer_set_monitor(window_.gobj(), output.monitor->gobj());
     gtk_layer_set_namespace(window_.gobj(), "waybar");
 
+    window.signal_map_event().connect_notify(sigc::mem_fun(*this, &GLSSurfaceImpl::onMap));
     window.signal_configure_event().connect_notify(
         sigc::mem_fun(*this, &GLSSurfaceImpl::onConfigure));
   }
@@ -62,6 +146,18 @@ struct GLSSurfaceImpl : public BarSurface, public sigc::trackable {
     gtk_layer_set_layer(window_.gobj(), layer);
   }
 
+  void setPassThrough(bool enable) override {
+    passthrough_ = enable;
+    auto gdk_window = window_.get_window();
+    if (gdk_window) {
+      Cairo::RefPtr<Cairo::Region> region;
+      if (enable) {
+        region = Cairo::Region::create();
+      }
+      gdk_window->input_shape_combine_region(region, 0, 0);
+    }
+  }
+
   void setPosition(const std::string_view& position) override {
     auto unanchored = GTK_LAYER_SHELL_EDGE_BOTTOM;
     vertical_ = false;
@@ -93,8 +189,11 @@ struct GLSSurfaceImpl : public BarSurface, public sigc::trackable {
   std::string  output_name_;
   uint32_t     width_;
   uint32_t     height_;
+  bool         passthrough_ = false;
   bool         vertical_ = false;
 
+  void onMap(GdkEventAny* ev) { setPassThrough(passthrough_); }
+
   void onConfigure(GdkEventConfigure* ev) {
     /*
      * GTK wants new size for the window.
@@ -182,6 +281,20 @@ struct RawSurfaceImpl : public BarSurface, public sigc::trackable {
     }
   }
 
+  void setPassThrough(bool enable) override {
+    passthrough_ = enable;
+    /* GTK overwrites any region changes applied directly to the wl_surface,
+     * thus the same GTK region API as in the GLS impl has to be used. */
+    auto gdk_window = window_.get_window();
+    if (gdk_window) {
+      Cairo::RefPtr<Cairo::Region> region;
+      if (enable) {
+        region = Cairo::Region::create();
+      }
+      gdk_window->input_shape_combine_region(region, 0, 0);
+    }
+  }
+
   void setPosition(const std::string_view& position) override {
     anchor_ = HORIZONTAL_ANCHOR | ZWLR_LAYER_SURFACE_V1_ANCHOR_TOP;
     if (position == "bottom") {
@@ -230,6 +343,7 @@ struct RawSurfaceImpl : public BarSurface, public sigc::trackable {
   uint32_t           height_ = 0;
   uint8_t            anchor_ = HORIZONTAL_ANCHOR | ZWLR_LAYER_SURFACE_V1_ANCHOR_TOP;
   bool               exclusive_zone_ = true;
+  bool               passthrough_ = false;
   struct bar_margins margins_;
 
   zwlr_layer_shell_v1_layer layer_ = ZWLR_LAYER_SHELL_V1_LAYER_BOTTOM;
@@ -262,6 +376,7 @@ struct RawSurfaceImpl : public BarSurface, public sigc::trackable {
 
     setSurfaceSize(width_, height_);
     setExclusiveZone(exclusive_zone_);
+    setPassThrough(passthrough_);
 
     commit();
     wl_display_roundtrip(client->wl_display);
@@ -359,7 +474,6 @@ waybar::Bar::Bar(struct waybar_output* w_output, const Json::Value& w_config)
     : output(w_output),
       config(w_config),
       window{Gtk::WindowType::WINDOW_TOPLEVEL},
-      layer_{bar_layer::BOTTOM},
       left_(Gtk::ORIENTATION_HORIZONTAL, 0),
       center_(Gtk::ORIENTATION_HORIZONTAL, 0),
       right_(Gtk::ORIENTATION_HORIZONTAL, 0),
@@ -370,15 +484,6 @@ waybar::Bar::Bar(struct waybar_output* w_output, const Json::Value& w_config)
   window.get_style_context()->add_class(output->name);
   window.get_style_context()->add_class(config["name"].asString());
   window.get_style_context()->add_class(config["position"].asString());
-  left_.get_style_context()->add_class("modules-left");
-  center_.get_style_context()->add_class("modules-center");
-  right_.get_style_context()->add_class("modules-right");
-
-  if (config["layer"] == "top") {
-    layer_ = bar_layer::TOP;
-  } else if (config["layer"] == "overlay") {
-    layer_ = bar_layer::OVERLAY;
-  }
 
   auto position = config["position"].asString();
 
@@ -390,6 +495,17 @@ waybar::Bar::Bar(struct waybar_output* w_output, const Json::Value& w_config)
     vertical = true;
   }
 
+  left_.get_style_context()->add_class("modules-left");
+  center_.get_style_context()->add_class("modules-center");
+  right_.get_style_context()->add_class("modules-right");
+
+  if (config["spacing"].isInt()) {
+    int spacing = config["spacing"].asInt();
+    left_.set_spacing(spacing);
+    center_.set_spacing(spacing);
+    right_.set_spacing(spacing);
+  }
+
   uint32_t height = config["height"].isUInt() ? config["height"].asUInt() : 0;
   uint32_t width = config["width"].isUInt() ? config["width"].asUInt() : 0;
 
@@ -450,14 +566,43 @@ waybar::Bar::Bar(struct waybar_output* w_output, const Json::Value& w_config)
     surface_impl_ = std::make_unique<RawSurfaceImpl>(window, *output);
   }
 
-  surface_impl_->setLayer(layer_);
-  surface_impl_->setExclusiveZone(true);
   surface_impl_->setMargins(margins_);
   surface_impl_->setPosition(position);
   surface_impl_->setSize(width, height);
 
+  /* Read custom modes if available */
+  if (auto modes = config.get("modes", {}); modes.isObject()) {
+    from_json(modes, configured_modes);
+  }
+
+  /* Update "default" mode with the global bar options */
+  from_json(config, configured_modes[MODE_DEFAULT]);
+
+  if (auto mode = config.get("mode", {}); mode.isString()) {
+    setMode(config["mode"].asString());
+  } else {
+    setMode(MODE_DEFAULT);
+  }
+
   window.signal_map_event().connect_notify(sigc::mem_fun(*this, &Bar::onMap));
 
+#if HAVE_SWAY
+  if (auto ipc = config["ipc"]; ipc.isBool() && ipc.asBool()) {
+    bar_id = Client::inst()->bar_id;
+    if (auto id = config["id"]; id.isString()) {
+      bar_id = id.asString();
+    }
+    if (bar_id.empty()) {
+      bar_id = DEFAULT_BAR_ID;
+    }
+    try {
+      _ipc_client = std::make_unique<BarIpcClient>(*this);
+    } catch (const std::exception& exc) {
+      spdlog::warn("Failed to open bar ipc connection: {}", exc.what());
+    }
+  }
+#endif
+
   setupWidgets();
   window.show_all();
 
@@ -472,6 +617,44 @@ waybar::Bar::Bar(struct waybar_output* w_output, const Json::Value& w_config)
   }
 }
 
+/* Need to define it here because of forward declared members */
+waybar::Bar::~Bar() = default;
+
+void waybar::Bar::setMode(const std::string_view& mode) {
+  using namespace std::literals::string_literals;
+
+  auto style = window.get_style_context();
+  /* remove styles added by previous setMode calls */
+  style->remove_class("mode-"s + last_mode_);
+
+  auto it = configured_modes.find(mode);
+  if (it != configured_modes.end()) {
+    last_mode_ = mode;
+    style->add_class("mode-"s + last_mode_);
+    setMode(it->second);
+  } else {
+    spdlog::warn("Unknown mode \"{}\" requested", mode);
+    last_mode_ = MODE_DEFAULT;
+    style->add_class("mode-"s + last_mode_);
+    setMode(configured_modes.at(MODE_DEFAULT));
+  }
+}
+
+void waybar::Bar::setMode(const struct bar_mode& mode) {
+  surface_impl_->setLayer(mode.layer);
+  surface_impl_->setExclusiveZone(mode.exclusive);
+  surface_impl_->setPassThrough(mode.passthrough);
+
+  if (mode.visible) {
+    window.get_style_context()->remove_class("hidden");
+    window.set_opacity(1);
+  } else {
+    window.get_style_context()->add_class("hidden");
+    window.set_opacity(0);
+  }
+  surface_impl_->commit();
+}
+
 void waybar::Bar::onMap(GdkEventAny*) {
   /*
    * Obtain a pointer to the custom layer surface for modules that require it (idle_inhibitor).
@@ -482,17 +665,7 @@ void waybar::Bar::onMap(GdkEventAny*) {
 
 void waybar::Bar::setVisible(bool value) {
   visible = value;
-  if (!visible) {
-    window.get_style_context()->add_class("hidden");
-    window.set_opacity(0);
-    surface_impl_->setLayer(bar_layer::BOTTOM);
-  } else {
-    window.get_style_context()->remove_class("hidden");
-    window.set_opacity(1);
-    surface_impl_->setLayer(layer_);
-  }
-  surface_impl_->setExclusiveZone(visible);
-  surface_impl_->commit();
+  setMode(visible ? MODE_DEFAULT : MODE_INVISIBLE);
 }
 
 void waybar::Bar::toggle() { setVisible(!visible); }
@@ -538,19 +711,7 @@ void waybar::Bar::setupAltFormatKeyForModuleList(const char* module_list_name) {
 }
 
 void waybar::Bar::handleSignal(int signal) {
-  for (auto& module : modules_left_) {
-    auto* custom = dynamic_cast<waybar::modules::Custom*>(module.get());
-    if (custom != nullptr) {
-      custom->refresh(signal);
-    }
-  }
-  for (auto& module : modules_center_) {
-    auto* custom = dynamic_cast<waybar::modules::Custom*>(module.get());
-    if (custom != nullptr) {
-      custom->refresh(signal);
-    }
-  }
-  for (auto& module : modules_right_) {
+  for (auto& module : modules_all_) {
     auto* custom = dynamic_cast<waybar::modules::Custom*>(module.get());
     if (custom != nullptr) {
       custom->refresh(signal);
@@ -558,19 +719,36 @@ void waybar::Bar::handleSignal(int signal) {
   }
 }
 
-void waybar::Bar::getModules(const Factory& factory, const std::string& pos) {
-  if (config[pos].isArray()) {
-    for (const auto& name : config[pos]) {
+void waybar::Bar::getModules(const Factory& factory, const std::string& pos, Gtk::Box* group = nullptr) {
+  auto module_list = group ? config[pos]["modules"] : config[pos];
+  if (module_list.isArray()) {
+    for (const auto& name : module_list) {
       try {
-        auto module = factory.makeModule(name.asString());
-        if (pos == "modules-left") {
-          modules_left_.emplace_back(module);
+        auto ref = name.asString();
+        AModule* module;
+
+        if (ref.compare(0, 6, "group/") == 0 && ref.size() > 6) {
+          auto group_module = new waybar::Group(ref, *this, config[ref]);
+          getModules(factory, ref, &group_module->box);
+          module = group_module;
+        } else {
+          module = factory.makeModule(ref);
         }
-        if (pos == "modules-center") {
-          modules_center_.emplace_back(module);
-        }
-        if (pos == "modules-right") {
-          modules_right_.emplace_back(module);
+
+        std::shared_ptr<AModule> module_sp(module);
+        modules_all_.emplace_back(module_sp);
+        if (group) {
+          group->pack_start(*module, false, false);
+        } else {
+          if (pos == "modules-left") {
+            modules_left_.emplace_back(module_sp);
+          }
+          if (pos == "modules-center") {
+            modules_center_.emplace_back(module_sp);
+          }
+          if (pos == "modules-right") {
+            modules_right_.emplace_back(module_sp);
+          }
         }
         module->dp.connect([module, &name] {
           try {
@@ -589,7 +767,11 @@ void waybar::Bar::getModules(const Factory& factory, const std::string& pos) {
 auto waybar::Bar::setupWidgets() -> void {
   window.add(box_);
   box_.pack_start(left_, false, false);
-  box_.set_center_widget(center_);
+  if (config["fixed-center"].isBool() ? config["fixed-center"].asBool() : true) {
+    box_.set_center_widget(center_);
+  } else {
+    box_.pack_start(center_, true, false);
+  }
   box_.pack_end(right_, false, false);
 
   // Convert to button code for every module that is used.
