@@ -12,6 +12,10 @@ waybar::modules::IdleInhibitor::IdleInhibitor(const std::string& id, const Bar&
       bar_(bar),
       idle_inhibitor_(nullptr),
       pid_(-1) {
+  if (waybar::Client::inst()->idle_inhibit_manager == nullptr) {
+    throw std::runtime_error("idle-inhibit not available");
+  }
+
   event_box_.add_events(Gdk::BUTTON_PRESS_MASK);
   event_box_.signal_button_press_event().connect(
       sigc::mem_fun(*this, &IdleInhibitor::handleToggle));
