@@ -1,5 +1,7 @@
 #include "modules/wlr/taskbar.hpp"
 
+#include "glibmm/error.h"
+#include "glibmm/fileutils.h"
 #include "glibmm/refptr.h"
 #include "util/format.hpp"
 
@@ -9,12 +11,15 @@
 #include <cstring>
 #include <memory>
 #include <sstream>
+#include <utility>
 
+#include <fmt/core.h>
 #include <gdkmm/monitor.h>
 
 #include <gtkmm/icontheme.h>
 
 #include <giomm/desktopappinfo.h>
+#include <gio/gdesktopappinfo.h>
 
 #include <spdlog/spdlog.h>
 
@@ -26,19 +31,19 @@ const std::string WHITESPACE = " \n\r\t\f\v";
 
 static std::string ltrim(const std::string& s)
 {
-	size_t start = s.find_first_not_of(WHITESPACE);
-	return (start == std::string::npos) ? "" : s.substr(start);
+    size_t start = s.find_first_not_of(WHITESPACE);
+    return (start == std::string::npos) ? "" : s.substr(start);
 }
 
 static std::string rtrim(const std::string& s)
 {
-	size_t end = s.find_last_not_of(WHITESPACE);
-	return (end == std::string::npos) ? "" : s.substr(0, end + 1);
+    size_t end = s.find_last_not_of(WHITESPACE);
+    return (end == std::string::npos) ? "" : s.substr(0, end + 1);
 }
 
 static std::string trim(const std::string& s)
 {
-	return rtrim(ltrim(s));
+    return rtrim(ltrim(s));
 }
 
 
@@ -64,14 +69,26 @@ static std::vector<std::string> search_prefix()
         } while(end != std::string::npos);
     }
 
+    std::string home_dir = std::getenv("HOME");
+    prefixes.push_back(home_dir + "/.local/share/");
+
     for (auto& p : prefixes)
         spdlog::debug("Using 'desktop' search path prefix: {}", p);
 
     return prefixes;
 }
 
-/* Method 1 - get the correct icon name from the desktop file */
-static std::string get_from_desktop_app_info(const std::string &app_id)
+static Glib::RefPtr<Gdk::Pixbuf> load_icon_from_file(std::string icon_path, int size)
+{
+    try {
+        auto pb = Gdk::Pixbuf::create_from_file(icon_path, size, size);
+        return pb;
+    } catch(...) {
+        return {};
+    }
+}
+
+static Glib::RefPtr<Gio::DesktopAppInfo> get_app_info_by_name(const std::string& app_id)
 {
     static std::vector<std::string> prefixes = search_prefix();
 
@@ -87,66 +104,128 @@ static std::string get_from_desktop_app_info(const std::string &app_id)
         ".desktop"
     };
 
-    Glib::RefPtr<Gio::DesktopAppInfo> app_info;
+	for (auto& prefix : prefixes) {
+		for (auto& folder : app_folders) {
+			for (auto& suffix : suffixes) {
+                    auto app_info_ = Gio::DesktopAppInfo::create_from_filename(prefix + folder + app_id + suffix);
+					if (!app_info_) {
+						continue;
+					}
 
-    for (auto& prefix : prefixes)
-        for (auto& folder : app_folders)
-            for (auto& suffix : suffixes)
-                if (!app_info)
-                    app_info = Gio::DesktopAppInfo::create_from_filename(prefix + folder + app_id + suffix);
+					return app_info_;
+			}
+		}
+	}
 
-    if (app_info && app_info->get_icon())
-        return app_info->get_icon()->to_string();
-
-    return "";
+	return {};
 }
 
-/* Method 2 - use the app_id and check whether there is an icon with this name in the icon theme */
-static std::string get_from_icon_theme(const Glib::RefPtr<Gtk::IconTheme>& icon_theme,
-        const std::string &app_id) {
+Glib::RefPtr<Gio::DesktopAppInfo> get_desktop_app_info(const std::string &app_id)
+{
+	auto app_info = get_app_info_by_name(app_id);
+	if (app_info) {
+		return app_info;
+	}
 
+    std::string desktop_file = "";
+
+    gchar*** desktop_list = g_desktop_app_info_search(app_id.c_str());
+    if (desktop_list != nullptr && desktop_list[0] != nullptr) {
+        for (size_t i=0; desktop_list[0][i]; i++) {
+            if (desktop_file == "") {
+                desktop_file = desktop_list[0][i];
+            } else {
+                auto tmp_info = Gio::DesktopAppInfo::create(desktop_list[0][i]);
+                auto startup_class = tmp_info->get_startup_wm_class();
+
+                if (startup_class == app_id) {
+                    desktop_file = desktop_list[0][i];
+                    break;
+                }
+            }
+        }
+        g_strfreev(desktop_list[0]);
+    }
+    g_free(desktop_list);
+
+    return get_app_info_by_name(desktop_file);
+}
+
+void Task::set_app_info_from_app_id_list(const std::string& app_id_list) {
+    std::string app_id;
+    std::istringstream stream(app_id_list);
+
+    /* Wayfire sends a list of app-id's in space separated format, other compositors
+     * send a single app-id, but in any case this works fine */
+    while (stream >> app_id)
+    {
+        app_info_ = get_desktop_app_info(app_id);
+		if (app_info_) {
+			return;
+		}
+
+		auto lower_app_id = app_id;
+		std::transform(lower_app_id.begin(), lower_app_id.end(), lower_app_id.begin(),
+				[](char c){ return std::tolower(c); });
+		app_info_ = get_desktop_app_info(lower_app_id);
+		if (app_info_) {
+			return;
+		}
+				
+        size_t start = 0, end = app_id.size();
+        start = app_id.rfind(".", end);
+        std::string app_name = app_id.substr(start+1, app_id.size());
+        app_info_ = get_desktop_app_info(app_name);
+		if (app_info_) {
+			return;
+		}
+
+        start = app_id.find("-");
+        app_name = app_id.substr(0, start);
+        app_info_ = get_desktop_app_info(app_name);
+	}
+}
+
+static std::string get_icon_name_from_icon_theme(const Glib::RefPtr<Gtk::IconTheme>& icon_theme,
+        const std::string &app_id)
+{
     if (icon_theme->lookup_icon(app_id, 24))
         return app_id;
 
     return "";
 }
 
-static bool image_load_icon(Gtk::Image& image, const Glib::RefPtr<Gtk::IconTheme>& icon_theme,
-        const std::string &app_id_list, int size)
+bool Task::image_load_icon(Gtk::Image& image, const Glib::RefPtr<Gtk::IconTheme>& icon_theme, Glib::RefPtr<Gio::DesktopAppInfo> app_info, int size)
 {
-    std::string app_id;
-    std::istringstream stream(app_id_list);
-    bool found = false;
+	std::string ret_icon_name = "unknown";
+	if (app_info) {
+		std::string icon_name = get_icon_name_from_icon_theme(icon_theme, app_info->get_startup_wm_class());
+		if (!icon_name.empty()) {
+			ret_icon_name = icon_name;
+		} else {
+			if (app_info->get_icon()) {
+				ret_icon_name = app_info->get_icon()->to_string();
+			}
+		}
+	}
 
-    /* Wayfire sends a list of app-id's in space separated format, other compositors
-     * send a single app-id, but in any case this works fine */
-    while (stream >> app_id)
-    {
-        auto lower_app_id = app_id;
-        std::transform(lower_app_id.begin(), lower_app_id.end(), lower_app_id.begin(),
-                [](char c){ return std::tolower(c); });
+	Glib::RefPtr<Gdk::Pixbuf> pixbuf;
 
-        std::string icon_name = get_from_icon_theme(icon_theme, app_id);
+	try {
+		pixbuf = icon_theme->load_icon(ret_icon_name, size, Gtk::ICON_LOOKUP_FORCE_SIZE);
+	} catch(...) {
+		if (Glib::file_test(ret_icon_name, Glib::FILE_TEST_EXISTS))
+			pixbuf = load_icon_from_file(ret_icon_name, size);
+		else
+			pixbuf = {};
+	}
 
-        if (icon_name.empty())
-            icon_name = get_from_icon_theme(icon_theme, lower_app_id);
-        if (icon_name.empty())
-            icon_name = get_from_desktop_app_info(app_id);
-        if (icon_name.empty())
-            icon_name = get_from_desktop_app_info(lower_app_id);
+	if (pixbuf) {
+		image.set(pixbuf);
+		return true;
+	}
 
-        if (icon_name.empty())
-            continue;
-
-        auto pixbuf = icon_theme->load_icon(icon_name, size, Gtk::ICON_LOOKUP_FORCE_SIZE);
-        if (pixbuf) {
-            image.set(pixbuf);
-            found = true;
-            break;
-        }
-    }
-
-    return found;
+    return false;
 }
 
 /* Task class implementation */
@@ -213,8 +292,7 @@ Task::Task(const waybar::Bar &bar, const Json::Value &config, Taskbar *tbar,
         struct zwlr_foreign_toplevel_handle_v1 *tl_handle, struct wl_seat *seat) :
     bar_{bar}, config_{config}, tbar_{tbar}, handle_{tl_handle}, seat_{seat},
     id_{global_id++},
-    content_{bar.vertical ? Gtk::ORIENTATION_VERTICAL : Gtk::ORIENTATION_HORIZONTAL, 0},
-    button_visible_{false}
+    content_{bar.vertical ? Gtk::ORIENTATION_VERTICAL : Gtk::ORIENTATION_HORIZONTAL, 0}
 {
     zwlr_foreign_toplevel_handle_v1_add_listener(handle_, &toplevel_handle_impl, this);
 
@@ -227,13 +305,15 @@ Task::Task(const waybar::Bar &bar, const Json::Value &config, Taskbar *tbar,
     content_.show();
     button_.add(content_);
 
-    with_icon_ = false;
     format_before_.clear();
     format_after_.clear();
 
     if (config_["format"].isString()) {
         /* The user defined a format string, use it */
         auto format = config_["format"].asString();
+		if (format.find("{name}") != std::string::npos) {
+			with_name_ = true;
+		}
 
         auto icon_pos = format.find("{icon}");
         if (icon_pos == 0) {
@@ -314,19 +394,54 @@ std::string Task::state_string(bool shortened) const
 void Task::handle_title(const char *title)
 {
     title_ = title;
+    hide_if_ignored();
+}
+
+void Task::hide_if_ignored()
+{
+    if (tbar_->ignore_list().count(app_id_) || tbar_->ignore_list().count(title_)) {
+        ignored_ = true;
+        if (button_visible_) {
+          auto output = gdk_wayland_monitor_get_wl_output(bar_.output->monitor->gobj());
+          handle_output_leave(output);
+        }
+    } else {
+        bool is_was_ignored = ignored_;
+        ignored_ = false;
+        if (is_was_ignored) {
+          auto output = gdk_wayland_monitor_get_wl_output(bar_.output->monitor->gobj());
+          handle_output_enter(output);
+        }
+    }
 }
 
 void Task::handle_app_id(const char *app_id)
 {
     app_id_ = app_id;
+    hide_if_ignored();
 
-    if (!with_icon_)
+	auto ids_replace_map = tbar_->app_ids_replace_map();
+	if (ids_replace_map.count(app_id_)) {
+		auto replaced_id = ids_replace_map[app_id_];
+		spdlog::debug(fmt::format("Task ({}) [{}] app_id was replaced with {}", id_, app_id_, replaced_id));
+		app_id_ = replaced_id;
+	}
+
+	if (!with_icon_ && !with_name_) {
         return;
+	}
+
+	set_app_info_from_app_id_list(app_id_);
+	name_ = app_info_ ? app_info_->get_display_name() : app_id;
+
+	if (!with_icon_) {
+		return;
+	}
 
     int icon_size = config_["icon-size"].isInt() ? config_["icon-size"].asInt() : 16;
     bool found = false;
     for (auto& icon_theme : tbar_->icon_themes()) {
-        if (image_load_icon(icon_, icon_theme, app_id_, icon_size)) {
+        if (image_load_icon(icon_, icon_theme, app_info_, icon_size)) {
             found = true;
             break;
         }
@@ -340,6 +455,11 @@ void Task::handle_app_id(const char *app_id)
 
 void Task::handle_output_enter(struct wl_output *output)
 {
+    if (ignored_) {
+      spdlog::debug("{} is ignored", repr());
+      return;
+    }
+
     spdlog::debug("{} entered output {}", repr(), (void*)output);
 
     if (!button_visible_ && (tbar_->all_outputs() || tbar_->show_output(output))) {
@@ -367,16 +487,16 @@ void Task::handle_output_leave(struct wl_output *output)
 void Task::handle_state(struct wl_array *state)
 {
     state_ = 0;
-    for (auto* entry = static_cast<uint32_t*>(state->data);
-         entry < static_cast<uint32_t*>(state->data) + state->size;
-         entry++) {
-        if (*entry == ZWLR_FOREIGN_TOPLEVEL_HANDLE_V1_STATE_MAXIMIZED)
+    size_t size = state->size / sizeof(uint32_t);
+    for (size_t i = 0; i < size; ++i) {
+        auto entry = static_cast<uint32_t*>(state->data)[i];
+        if (entry == ZWLR_FOREIGN_TOPLEVEL_HANDLE_V1_STATE_MAXIMIZED)
             state_ |= MAXIMIZED;
-        if (*entry == ZWLR_FOREIGN_TOPLEVEL_HANDLE_V1_STATE_MINIMIZED)
+        if (entry == ZWLR_FOREIGN_TOPLEVEL_HANDLE_V1_STATE_MINIMIZED)
             state_ |= MINIMIZED;
-        if (*entry == ZWLR_FOREIGN_TOPLEVEL_HANDLE_V1_STATE_ACTIVATED)
+        if (entry == ZWLR_FOREIGN_TOPLEVEL_HANDLE_V1_STATE_ACTIVATED)
             state_ |= ACTIVE;
-        if (*entry == ZWLR_FOREIGN_TOPLEVEL_HANDLE_V1_STATE_FULLSCREEN)
+        if (entry == ZWLR_FOREIGN_TOPLEVEL_HANDLE_V1_STATE_FULLSCREEN)
             state_ |= FULLSCREEN;
     }
 }
@@ -477,19 +597,22 @@ void Task::update()
 {
     bool markup = config_["markup"].isBool() ? config_["markup"].asBool() : false;
     std::string title = title_;
+    std::string name = name_;
     std::string app_id = app_id_;
     if (markup) {
         title = Glib::Markup::escape_text(title);
+        name = Glib::Markup::escape_text(name);
         app_id = Glib::Markup::escape_text(app_id);
     }
     if (!format_before_.empty()) {
         auto txt = fmt::format(format_before_,
                     fmt::arg("title", title),
+                    fmt::arg("name", name),
                     fmt::arg("app_id", app_id),
                     fmt::arg("state", state_string()),
                     fmt::arg("short_state", state_string(true))
                 );
-        if (markup) 
+        if (markup)
             text_before_.set_markup(txt);
         else
             text_before_.set_label(txt);
@@ -498,11 +621,12 @@ void Task::update()
     if (!format_after_.empty()) {
         auto txt = fmt::format(format_after_,
                     fmt::arg("title", title),
+                    fmt::arg("name", name),
                     fmt::arg("app_id", app_id),
                     fmt::arg("state", state_string()),
                     fmt::arg("short_state", state_string(true))
                 );
-        if (markup) 
+        if (markup)
             text_after_.set_markup(txt);
         else
             text_after_.set_label(txt);
@@ -512,11 +636,12 @@ void Task::update()
     if (!format_tooltip_.empty()) {
         auto txt = fmt::format(format_tooltip_,
                     fmt::arg("title", title),
+                    fmt::arg("name", name),
                     fmt::arg("app_id", app_id),
                     fmt::arg("state", state_string()),
                     fmt::arg("short_state", state_string(true))
                 );
-        if (markup) 
+        if (markup)
             button_.set_tooltip_markup(txt);
         else
             button_.set_tooltip_text(txt);
@@ -584,7 +709,7 @@ static const wl_registry_listener registry_listener_impl = {
     .global_remove = handle_global_remove
 };
 
-Taskbar::Taskbar(const std::string &id, const waybar::Bar &bar, const Json::Value &config) 
+Taskbar::Taskbar(const std::string &id, const waybar::Bar &bar, const Json::Value &config)
     : waybar::AModule(config, "taskbar", id, false, false),
       bar_(bar),
       box_{bar.vertical ? Gtk::ORIENTATION_VERTICAL : Gtk::ORIENTATION_HORIZONTAL, 0},
@@ -631,14 +756,43 @@ Taskbar::Taskbar(const std::string &id, const waybar::Bar &bar, const Json::Valu
 
         icon_themes_.push_back(it);
     }
+
+    // Load ignore-list
+    if (config_["ignore-list"].isArray()) {
+        for (auto& app_name : config_["ignore-list"]) {
+          ignore_list_.emplace(app_name.asString());
+        }
+    }
+
+    // Load app_id remappings
+    if (config_["app_ids-mapping"].isObject()) {
+		const Json::Value& mapping = config_["app_ids-mapping"];
+		const std::vector<std::string> app_ids = config_["app_ids-mapping"].getMemberNames();
+        for (auto& app_id : app_ids) {
+          app_ids_replace_map_.emplace(app_id, mapping[app_id].asString());
+        }
+    }
+
     icon_themes_.push_back(Gtk::IconTheme::get_default());
 }
 
 Taskbar::~Taskbar()
 {
     if (manager_) {
-        zwlr_foreign_toplevel_manager_v1_destroy(manager_);
-        manager_ = nullptr;
+        struct wl_display *display = Client::inst()->wl_display;
+        /*
+         * Send `stop` request and wait for one roundtrip.
+         * This is not quite correct as the protocol encourages us to wait for the .finished event,
+         * but it should work with wlroots foreign toplevel manager implementation.
+         */
+        zwlr_foreign_toplevel_manager_v1_stop(manager_);
+        wl_display_roundtrip(display);
+
+        if (manager_) {
+            spdlog::warn("Foreign toplevel manager destroyed before .finished event");
+            zwlr_foreign_toplevel_manager_v1_destroy(manager_);
+            manager_ = nullptr;
+        }
     }
 }
 
@@ -750,9 +904,10 @@ bool Taskbar::all_outputs() const
     return config_["all-outputs"].isBool() && config_["all-outputs"].asBool();
 }
 
-std::vector<Glib::RefPtr<Gtk::IconTheme>> Taskbar::icon_themes() const
-{
-    return icon_themes_;
-}
+const std::vector<Glib::RefPtr<Gtk::IconTheme>>& Taskbar::icon_themes() const { return icon_themes_; }
+
+const std::unordered_set<std::string>& Taskbar::ignore_list() const { return ignore_list_; }
+
+const std::map<std::string, std::string>& Taskbar::app_ids_replace_map() const { return app_ids_replace_map_; }
 
 } /* namespace waybar::modules::wlr */
